try:
  import cupy as cp
except ModuleNotFoundError:
  raise ModuleNotFoundError("cupy is not installed, please visit https://pypi.org/project/cupy/")

from . import kmeans
from . import kernels

from .CustomModule import CustomModule

from .PQ import PQ
from .MPQ import MPQ
from .SQ import SQ
from .IVFPQBase import IVFPQBase
from .IVFPQTopk import IVFPQTopk

from .IVFPQ import IVFPQ
from .IVFPQR import IVFPQR
from .IVFMPQ import IVFMPQ



import torch
import torch.nn as nn

class CustomModule(nn.Module):
  def __init__(self):
    super(CustomModule, self).__init__()

  def load_state_dict(self, state_dict):
    for k, v in state_dict.items():
      if "." not in k:
        assert hasattr(self, k), f"attribute {k} does not exist"
        delattr(self, k)
        self.register_buffer(k, v)
        
    for name, module in self.named_children():
      sd = {k.replace(name+".", "") : v for k, v in state_dict.items() if k.startswith(name+".")}
      module.load_state_dict(sd)
      # else:
      #   module = k.split('.')[0]
      #   assert hasattr(self, module), f"module {module} does not exist"
      #   module = getattr(self, module)
      #   k = ".".join(k.split('.')[1:])
      #   module.load_state_dict( {k: v} )